use std::{any::Any, sync::Arc};

use actix_web::{HttpResponse, Responder, web};

use crate::{
    adapter::delivery::post_controller::PostController, application::error::post_error::PostError,
};

pub fn configure_post_routes(cfg: &mut web::ServiceConfig) {
    cfg.service(web::resource("/post_info").route(web::get().to(get_all_post_info)));
    cfg.service(web::resource("/post/{id}").route(web::get().to(get_full_post)));
}

async fn get_all_post_info(post_controller: web::Data<Arc<dyn PostController>>) -> impl Responder {
    let result = post_controller.get_all_post_info().await;

    match result {
        Ok(post_info_list) => HttpResponse::Ok().json(post_info_list),
        Err(e) => {
            log::error!("{e:?}");
            HttpResponse::InternalServerError().finish()
        }
    }
}

async fn get_full_post(
    post_controller: web::Data<Arc<dyn PostController>>,
    path: web::Path<i32>,
) -> impl Responder {
    let id = path.into_inner();
    let result = post_controller.get_full_post(id).await;

    match result {
        Ok(post) => HttpResponse::Ok().json(post),
        Err(e) => {
            if e.type_id() == PostError::NotFound.type_id() {
                HttpResponse::NotFound().finish()
            } else {
                log::error!("{e:?}");
                HttpResponse::InternalServerError().finish()
            }
        }
    }
}
